@@ -25,48 +25,20 @@ schema_basedir = os.path.dirname(os.path.abspath(__file__))
 re._MAXCACHE = 2048
 
 
-class tagged_list(list):
-
-    tags = {u'!u8': 8, u'!u16': 16, u'!u32': 32, u'!u64': 64}
-
-    def __init__(self, int_list, tag, tags=tags):
-        int_list = [sized_int(i, size=tags[tag]) for i in int_list]
-        super().__init__(int_list)
-
-    @staticmethod
-    def constructor(loader, node):
-        return tagged_list(loader.construct_sequence(node), node.tag)
-
-
 class sized_int(int):
     def __new__(cls, value, *args, **kwargs):
         return int.__new__(cls, value)
 
-    def __init__(self, value, size=32, phandle=False):
+    def __init__(self, value, size=32):
         self.size = size
-        self.phandle = phandle
-
-    @staticmethod
-    def constructor(loader, node):
-        return sized_int(loader.construct_yaml_int(node), phandle=True)
 
 
 rtyaml = ruamel.yaml.YAML(typ='rt')
 rtyaml.allow_duplicate_keys = False
 rtyaml.preserve_quotes = True
-rtyaml.Constructor.add_constructor(u'!u8', tagged_list.constructor)
-rtyaml.Constructor.add_constructor(u'!u16', tagged_list.constructor)
-rtyaml.Constructor.add_constructor(u'!u32', tagged_list.constructor)
-rtyaml.Constructor.add_constructor(u'!u64', tagged_list.constructor)
-rtyaml.Constructor.add_constructor(u'!phandle', sized_int.constructor)
 
 yaml = ruamel.yaml.YAML(typ='safe')
 yaml.allow_duplicate_keys = False
-yaml.Constructor.add_constructor(u'!u8', tagged_list.constructor)
-yaml.Constructor.add_constructor(u'!u16', tagged_list.constructor)
-yaml.Constructor.add_constructor(u'!u32', tagged_list.constructor)
-yaml.Constructor.add_constructor(u'!u64', tagged_list.constructor)
-yaml.Constructor.add_constructor(u'!phandle', sized_int.constructor)
 
 
 def path_to_obj(tree, path):
@@ -215,11 +187,6 @@ def _is_matrix_schema(subschema):
     return False
 
 
-known_variable_matrix_props = {
-    'fsl,pins',
-}
-
-
 # If we have a matrix with variable inner and outer dimensions, then drop the dimensions
 # because we have no way to reconstruct them.
 def _fixup_int_matrix(propname, subschema):
@@ -229,8 +196,7 @@ def _fixup_int_matrix(propname, subschema):
     outer_dim = _get_array_range(subschema)
     inner_dim = _get_array_range(subschema.get('items', {}))
 
-    if propname in known_variable_matrix_props or \
-       (outer_dim[0] != outer_dim[1] and inner_dim[0] != inner_dim[1]):
+    if outer_dim[0] != outer_dim[1] and inner_dim[0] != inner_dim[1]:
         subschema.pop('items', None)
         subschema.pop('maxItems', None)
         subschema.pop('minItems', None)
@@ -469,6 +435,12 @@ def fixup_schema(schema):
     fixup_sub_schema(schema, True)
 
 
+known_variable_matrix_props = {
+    'fsl,pins',
+    'qcom,board-id'
+}
+
+
 def fixup_sub_schema(schema, is_prop):
     if not isinstance(schema, dict):
         return
@@ -495,6 +467,13 @@ def fixup_sub_schema(schema, is_prop):
             continue
 
         for prop in v:
+            if prop in known_variable_matrix_props and isinstance(v[prop], dict):
+                ref = v[prop].pop('$ref', None)
+                schema[k][prop] = {}
+                if ref:
+                    schema[k][prop]['$ref'] = ref
+                continue
+
             walk_properties(prop, v[prop])
             # Recurse to check for {properties,patternProperties} in each prop
             fixup_sub_schema(v[prop], True)
@@ -503,7 +482,7 @@ def fixup_sub_schema(schema, is_prop):
 
 
 def fixup_node_props(schema):
-    if not {'properties', 'patternProperties', 'unevaluatedProperties'} & schema.keys():
+    if not {'properties', 'patternProperties', 'unevaluatedProperties', 'additionalProperties'} & schema.keys():
         return
     if ('additionalProperties' in schema and schema['additionalProperties'] is True) or \
        ('unevaluatedProperties' in schema and schema['unevaluatedProperties'] is True):
@@ -514,6 +493,11 @@ def fixup_node_props(schema):
     schema['properties'].setdefault('status', True)
     schema['properties'].setdefault('secure-status', True)
     schema['properties'].setdefault('$nodename', True)
+    schema['properties'].setdefault('bootph-pre-sram', True)
+    schema['properties'].setdefault('bootph-verify', True)
+    schema['properties'].setdefault('bootph-pre-ram', True)
+    schema['properties'].setdefault('bootph-some-ram', True)
+    schema['properties'].setdefault('bootph-all', True)
 
     keys = list()
     if 'properties' in schema:
@@ -720,22 +704,18 @@ def get_undocumented_compatibles(compatible_list):
 def process_schema(filename):
     try:
         schema = load_schema(filename)
-    except:
+    except ruamel.yaml.YAMLError:
         print(filename + ": ignoring, error parsing file", file=sys.stderr)
         return
 
     # Check that the validation schema is valid
     try:
-        DTValidator.check_schema(schema)
+        DTValidator.check_schema(schema, strict=False)
     except jsonschema.SchemaError as exc:
         print(filename + ": ignoring, error in schema: " + ': '.join(str(x) for x in exc.path),
               file=sys.stderr)
         #print(exc.message)
         return
-    except:
-        print(filename + ": ignoring, unknown error in schema (not a schema?)",
-              file=sys.stderr)
-        return
 
     if 'select' not in schema:
         print(filename + ": warning: no 'select' found in schema found", file=sys.stderr)
@@ -753,7 +733,7 @@ def process_schemas(schema_paths, core_schema=True):
         if not os.path.isfile(filename):
             continue
         sch = process_schema(os.path.abspath(filename))
-        if not sch:
+        if not sch or '$id' not in sch:
             continue
         if sch['$id'] in schemas:
             print(os.path.abspath(filename) + ": duplicate '$id' value '" + sch['$id'] + "'", file=sys.stderr)
@@ -826,11 +806,11 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
     # We only support local refs
     if '$ref' in subschema and subschema['$ref'].startswith('#/'):
         sch_path = subschema['$ref'].split('/')[1:]
-        subschema = schema
+        tmp_subschema = schema
         for p in sch_path:
-            subschema = subschema[p]
-        #print(propname, sch_path, subschema, file=sys.stderr)
-        _extract_prop_type(props, schema, propname, subschema, is_pattern)
+            tmp_subschema = tmp_subschema[p]
+        #print(propname, sch_path, tmp_subschema, file=sys.stderr)
+        _extract_prop_type(props, schema, propname, tmp_subschema, is_pattern)
 
     for k in subschema.keys() & {'allOf', 'oneOf', 'anyOf'}:
         for v in subschema[k]:
@@ -841,7 +821,8 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
     new_prop = {}
     prop_type = None
 
-    if 'type' in subschema and subschema['type'] == 'object':
+    if ('type' in subschema and subschema['type'] == 'object') or \
+        subschema.keys() & {'properties', 'patternProperties', 'additionalProperties'}:
         prop_type = 'node'
     else:
         try:
@@ -856,9 +837,14 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
                     # implicit string type
                     prop_type = 'string-array'
                 elif not (isinstance(items, list) and len(items) == 1 and \
-                    'items' in items and isinstance(items['items'], list) and len(items['items']) == 1) and \
-                    unit_types_re.search(propname):
-                    prop_type = 'uint32-matrix'
+                     'items' in items and isinstance(items['items'], list) and len(items['items']) == 1):
+                    # Keep in sync with property-units.yaml
+                    if re.search('-microvolt$', propname):
+                        prop_type = 'int32-matrix'
+                    elif re.search('(^(?!opp)).*-hz$', propname):
+                        prop_type = 'uint32-matrix'
+                    else:
+                        prop_type = None
                 else:
                     prop_type = None
             elif '$ref' in subschema and re.search(r'\.yaml#?$', subschema['$ref']):
@@ -896,13 +882,11 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
                 p['dim'] = _merge_dim(p['dim'], dim)
             return
         if p['type'].startswith(prop_type):
-            # Already have the same or looser type
+            # Already have the same or looser type, just record the $id
+            new_prop = None
             if schema['$id'] not in p['$id']:
                 p['$id'] += [schema['$id']]
-                # Descend into child schemas if we haven't seen this node already
-                if prop_type == 'node':
-                    break
-            return
+            break
         elif p['type'] in prop_type:
             # Replace scalar type with array type
             new_prop['$id'] += p['$id']
@@ -912,7 +896,8 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
     if dup_prop:
         props[propname].remove(dup_prop)
 
-    props[propname] += [new_prop]
+    if new_prop:
+        props[propname] += [new_prop]
 
     if subschema.keys() & {'properties', 'patternProperties', 'additionalProperties'}:
         _extract_subschema_types(props, schema, subschema)
@@ -1087,6 +1072,8 @@ def set_schemas(schema_files, core_schema=True):
         if isinstance(schema_cache, list):
             d = {}
             for sch in schema_cache:
+                if not isinstance(sch, dict):
+                    return None
                 d[sch['$id']] = sch
             schema_cache = d
 
@@ -1209,7 +1196,13 @@ class DTValidator():
 
     @classmethod
     def iter_schema_errors(cls, schema):
-        meta_schema = cls.resolver.resolve_from_url(schema['$schema'])
+        try:
+            meta_schema = cls.resolver.resolve_from_url(schema['$schema'])
+        except (KeyError, TypeError, jsonschema.RefResolutionError, jsonschema.SchemaError):
+            error = jsonschema.SchemaError("Missing or invalid $schema keyword")
+            error.linecol = (-1,-1)
+            yield error
+            return
         val = cls.DTVal(meta_schema, resolver=cls.resolver)
         for error in val.iter_errors(schema):
             cls.annotate_error(error, meta_schema, error.schema_path)
@@ -1218,9 +1211,6 @@ class DTValidator():
 
     def iter_errors(self, instance, _schema=None):
         for error in self.validator.iter_errors(instance, _schema):
-            error.linecol = get_line_col(instance, error.path)
-            error.note = None
-            error.schema_file = None
             yield error
 
     def validate(self, *args, **kwargs):
@@ -1232,8 +1222,16 @@ class DTValidator():
         return error is None
 
     @classmethod
-    def check_schema(cls, schema):
-        meta_schema = cls.resolver.resolve_from_url(schema['$schema'])
+    def check_schema(cls, schema, strict=True):
+        """
+        Test if schema is valid and apply fixups
+        'strict' determines whether the full DT meta-schema is used or just the draft7 meta-schema
+        """
+        if strict:
+            meta_schema = cls.resolver.resolve_from_url(schema['$schema'])
+        else:
+            # Using the draft7 metaschema because 2019-09 with $recursiveRef seems broken
+            meta_schema = jsonschema.Draft7Validator.META_SCHEMA
         val = cls.DTVal(meta_schema, resolver=cls.resolver)
         for error in val.iter_errors(schema):
             raise jsonschema.SchemaError.create_from(error)
@@ -1252,6 +1250,8 @@ class DTValidator():
 
     @classmethod
     def check_schema_refs(self, filename, schema):
+        if '$id' not in schema:
+            return
         scope = schema['$id']
         if scope:
             self.resolver.push_scope(scope)
@@ -1307,7 +1307,7 @@ class DTValidator():
 def format_error(filename, error, prefix="", nodename=None, verbose=False):
     src = prefix + os.path.abspath(filename) + ':'
 
-    if error.linecol[0] >= 0:
+    if hasattr(error, 'linecol') and error.linecol[0] >= 0:
         src = src + '%i:%i: ' % (error.linecol[0]+1, error.linecol[1]+1)
     else:
         src += ' '
@@ -1323,6 +1323,8 @@ def format_error(filename, error, prefix="", nodename=None, verbose=False):
     #print(error.__dict__)
     if verbose:
         msg = str(error)
+    elif not error.schema_path:
+        msg = error.message
     elif error.context:
         # An error on a conditional will have context with sub-errors
         msg = "'" + error.schema_path[-1] + "' conditional failed, one must be fixed:"
@@ -1342,10 +1344,10 @@ def format_error(filename, error, prefix="", nodename=None, verbose=False):
     else:
         msg = error.message
 
-    if error.note:
+    if hasattr(error, 'note') and error.note:
         msg += '\n\t' + prefix + 'hint: ' + error.note
 
-    if error.schema_file:
+    if hasattr(error, 'schema_file') and error.schema_file:
         msg += '\n\t' + prefix + 'from schema $id: ' + error.schema_file
 
     return src + msg
