@@ -132,21 +132,6 @@ class TestDTValidate(unittest.TestCase):
             if isinstance(value, dict):
                 self.check_subtree(name, value, fail)
 
-    def test_dt_yaml_validation(self):
-        '''Test that all DT files under ./test/ validate against the DT schema (YAML)'''
-        for filename in glob.iglob('test/*.dts'):
-            with self.subTest(schema=filename):
-                expect_fail = "-fail" in filename
-                tmpfile = tempfile.NamedTemporaryFile()
-                # The test files have lots of expected warnings, so send stderr to /dev/null
-                res = subprocess.run(['dtc', '-Oyaml', filename], stdout=tmpfile, stderr=subprocess.PIPE)
-                self.assertEqual(res.returncode, 0, msg='dtc failed:\n' + res.stderr.decode())
-
-                testtree = dtschema.load(tmpfile.name)[0]
-                for name,value in testtree.items():
-                    if isinstance(value, dict):
-                        self.check_node(name, value, expect_fail)
-
     def test_dtb_validation(self):
         '''Test that all DT files under ./test/ validate against the DT schema (DTB)'''
         for filename in glob.iglob('test/*.dts'):
